# Project:  MapServer
# Purpose:  xUnit style Python mapscript test of multi-threading
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import os
import threading
import unittest

import mapscript

from .testing import INCOMING, TESTMAPFILE


def draw_map(name, save=0):

    # print("making map in thread %s" % (name))
    mo = mapscript.mapObj(TESTMAPFILE)
    im = mo.draw()
    if save:
        im.save("threadtest_%s.png" % (name))


def trigger_exception(name):

    # print("triggering exception in thread %s" % (name))
    mo = mapscript.mapObj(TESTMAPFILE)
    try:
        mo.setExtent(1, 50, -1, 51)
        raise Exception("We expected a MapServer exception")
    except mapscript.MapServerError:
        pass


class MultipleThreadsTestCase(unittest.TestCase):
    def testDrawMultiThreads(self):
        """map drawing with multiple threads"""

        workers = []
        for i in range(10):
            name = "d%d" % (i)
            thread = threading.Thread(target=draw_map, name=name, args=(name, 1))
            workers.append(thread)
            thread.start()

    def testExceptionsMultiThreads(self):
        """mapserver exceptions behave with multiple threads"""

        workers = []
        for i in range(10):
            name = "e%d" % (i)
            thread = threading.Thread(target=trigger_exception, name=name, args=(name,))
            workers.append(thread)
            thread.start()

    def testExceptionContainmentMultiThreads(self):
        """mapserver exceptions should be contained to a thread"""
        num = 100
        workers = []

        # Trigger an exception in the first started thread
        for i in range(0, 1):
            name = "c%d" % (i)
            thread = threading.Thread(target=trigger_exception, name=name, args=(name,))
            workers.append(thread)

        # Draw normally
        for i in range(1, num):
            name = "c%d" % (i)
            thread = threading.Thread(target=draw_map, name=name, args=(name,))
            workers.append(thread)

        # Start all threads
        for i in range(num):
            workers[i].start()


def draw_map_wfs(name, save=0):

    # print("making map in thread %s" % (name))
    mo = mapscript.mapObj(TESTMAPFILE)

    # WFS layer
    lo = mapscript.layerObj()
    lo.name = "cheapo_wfs"
    lo.setProjection("+init=epsg:4326")
    lo.connectiontype = mapscript.MS_WFS
    lo.connection = "http://zcologia.com:9001/mapserver/members/features.rpy?"
    lo.metadata.set("wfs_service", "WFS")
    lo.metadata.set("wfs_typename", "users")
    lo.metadata.set("wfs_version", "1.0.0")
    lo.type = mapscript.MS_LAYER_POINT
    lo.status = mapscript.MS_DEFAULT
    lo.labelitem = "zco:mid"

    so1 = mapscript.styleObj()
    so1.color.setHex("#FFFFFF")
    so1.size = 9
    so1.symbol = 1  # mo.symbolset.index('circle')

    so2 = mapscript.styleObj()
    so2.color.setHex("#333333")
    so2.size = 7
    so2.symbol = 1  # mo.symbolset.index('circle')

    co = mapscript.classObj()
    co.label.type = mapscript.MS_BITMAP
    co.label.size = mapscript.MS_SMALL
    co.label.color.setHex("#000000")
    co.label.outlinecolor.setHex("#FFFFFF")
    co.label.position = mapscript.MS_AUTO

    co.insertStyle(so1)
    co.insertStyle(so2)
    lo.insertClass(co)
    mo.insertLayer(lo)

    if not mo.web.imagepath:
        mo.web.imagepath = os.environ.get("TEMP", None) or INCOMING
    mo.debug = mapscript.MS_ON
    im = mo.draw()
    if save:
        im.save("threadtest_wfs_%s.png" % (name))


def draw_map_wms(name, save=0):

    # print("making map in thread %s" % (name))
    mo = mapscript.mapObj(TESTMAPFILE)
    # WMS layer
    lo = mapscript.layerObj()
    lo.name = "world_latlong"
    lo.setProjection("+init=epsg:4326")
    lo.connectiontype = mapscript.MS_WMS
    lo.connection = "https://demo.mapserver.org/cgi-bin/msautotest?"
    lo.metadata.set("wms_service", "WMS")
    lo.metadata.set("wms_server_version", "1.3.0")
    lo.metadata.set("wms_name", "world_latlong")
    lo.metadata.set("wms_format", "image/jpeg")
    lo.type = mapscript.MS_LAYER_RASTER
    lo.status = mapscript.MS_DEFAULT
    lo.debug = mapscript.MS_ON
    mo.insertLayer(lo)

    if not mo.web.imagepath:
        mo.web.imagepath = os.environ.get("TEMP", None) or INCOMING
    mo.debug = mapscript.MS_ON
    mo.selectOutputFormat("image/jpeg")
    im = mo.draw()
    if save:
        im.save("threadtest_wms_%s.jpg" % (name))


class OWSRequestTestCase(unittest.TestCase):

    # def testDrawWFS(self):
    #    workers = []
    #    for i in range(10):
    #        name = 'd%d' % (i)
    #        thread = threading.Thread(target=draw_map_wfs, name=name, args=(name, 1))
    #        workers.append(thread)
    #        thread.start()

    def testDrawWMS(self):
        workers = []
        for i in range(10):
            name = "d%d" % (i)
            thread = threading.Thread(target=draw_map_wms, name=name, args=(name, 1))
            workers.append(thread)
            thread.start()


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of PostGIS Layer
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================
#
# Execute this module as a script from mapserver/mapscript/python
#
#     python tests/cases/pgtest.py -v
#
# NOTE: these tests use a fixture included in mapserver/tests. To produce
# the testing database, follow these steps:
#
# $ cd mapserver/tests
# $ shp2pgsql -s 4326 -I polygon.shp postgis.polygon > polygon.sql
# $ su postgres
# $ make -f makefile_postgis
#
# ===========================================================================

import unittest

import mapscript

PG_CONNECTION_STRING = "dbname=mapserver_test user=postgres"


class TableTest(unittest.TestCase):
    def setUp(self):
        self.mo = mapscript.mapObj()
        lo = mapscript.layerObj()
        lo.name = "pg_layer"
        lo.type = mapscript.MS_LAYER_POLYGON
        lo.connectiontype = mapscript.MS_POSTGIS
        lo.connection = PG_CONNECTION_STRING
        lo.data = "the_geom from polygon"
        li = self.mo.insertLayer(lo)
        self.lo = self.mo.getLayer(li)

    def test_getfeature(self):
        self.lo.open()
        f = self.lo.getFeature(1, -1)
        atts = [f.getValue(i) for i in range(self.lo.numitems)]
        self.lo.close()
        self.assertEqual(atts, ["1", "1", "A Polygon"], atts)


class ViewTest(unittest.TestCase):
    def setUp(self):
        self.mo = mapscript.mapObj()
        lo = mapscript.layerObj()
        lo.name = "pg_sub_layer"
        lo.type = mapscript.MS_LAYER_POLYGON
        lo.connectiontype = mapscript.MS_POSTGIS
        lo.connection = PG_CONNECTION_STRING
        lo.data = "the_geom from polygon_v using unique gid using srid=4326"
        li = self.mo.insertLayer(lo)
        self.lo = self.mo.getLayer(li)

    def test_getfeature(self):
        self.lo.open()
        f = self.lo.getFeature(1, -1)
        atts = [f.getValue(i) for i in range(self.lo.numitems)]
        self.lo.close()
        self.assertEqual(atts, ["1", "1", "A Polygon"], atts)


class SubSelectTest(unittest.TestCase):
    def setUp(self):
        self.mo = mapscript.mapObj()
        lo = mapscript.layerObj()
        lo.name = "pg_sub_layer"
        lo.type = mapscript.MS_LAYER_POLYGON
        lo.connectiontype = mapscript.MS_POSTGIS
        lo.connection = PG_CONNECTION_STRING
        lo.data = "the_geom from (select * from polygon) as foo using unique gid using srid=4326"
        li = self.mo.insertLayer(lo)
        self.lo = self.mo.getLayer(li)

    def test_getfeature(self):
        self.lo.open()
        f = self.lo.getFeature(1, -1)
        atts = [f.getValue(i) for i in range(self.lo.numitems)]
        self.lo.close()
        self.assertEqual(atts, ["1", "1", "A Polygon"], atts)


class SubSelectNoSRIDTest(unittest.TestCase):
    """SRID should be diagnosed from the selection"""

    def setUp(self):
        self.mo = mapscript.mapObj()
        lo = mapscript.layerObj()
        lo.name = "pg_sub_layer"
        lo.type = mapscript.MS_LAYER_POLYGON
        lo.connectiontype = mapscript.MS_POSTGIS
        lo.connection = PG_CONNECTION_STRING
        lo.data = "the_geom from (select * from polygon) as foo using unique gid"
        li = self.mo.insertLayer(lo)
        self.lo = self.mo.getLayer(li)

    def test_getfeature(self):
        self.lo.open()
        f = self.lo.getFeature(1, -1)
        atts = [f.getValue(i) for i in range(self.lo.numitems)]
        self.lo.close()
        self.assertEqual(atts, ["1", "1", "A Polygon"], atts)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

"""
Simple example to read a csv file and reproject point x/y data

Usage:

project_csv.py cities.csv 2 1 EPSG:4326 EPSG:3857

"""

import csv
import sys
from io import open

import mapscript


def main(input_file, x_field_idx, y_field_idx, input_proj, output_proj):

    # set input and output projections
    proj_in = mapscript.projectionObj(input_proj)
    proj_out = mapscript.projectionObj(output_proj)

    # open file
    with open(input_file, encoding="utf-8") as f:
        # read csv
        csv_in = csv.reader(f)
        headers = next(csv_in)

        # setup output
        csv_out = csv.writer(sys.stdout)
        csv_out.writerow(headers)

        for row in csv_in:
            # set pointObj
            point = mapscript.pointObj(float(row[x_field_idx]), float(row[y_field_idx]))
            # project
            point.project(proj_in, proj_out)

            # update with reprojected coordinates
            row[x_field_idx] = point.x
            row[y_field_idx] = point.y

            csv_out.writerow(row)


def usage():
    """
    Display usage if program is used incorrectly
    """
    print(
        "Syntax: %s <csvfile> <x_col> <y_col> <epsg_code_in> <epsg_code_out>"
        % sys.argv[0]
    )
    sys.exit(2)


# check input parameters

if len(sys.argv) != 6:
    usage()


input_file = sys.argv[1]

# set x and y indices

x_field_idx = int(sys.argv[2])
y_field_idx = int(sys.argv[3])

# get projection codes

input_proj = "init=" + sys.argv[4].lower()
output_proj = "init=" + sys.argv[5].lower()
main(input_file, x_field_idx, y_field_idx, input_proj, output_proj)



# Project:  MapServer
# Purpose:  xUnit style Python mapscript testing utilities
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================
#
# Purpose of this module is to export the locally built mapscript module
# prior to installation, do some name normalization that allows testing of
# the so-called next generation class names, and define some classes
# useful to many test cases.
#
# ===========================================================================

import os
import tempfile
import unittest

import mapscript

# define the path to mapserver test data
TESTS_PATH = os.path.abspath(os.path.join(os.path.dirname(__file__), "data"))
TESTMAPFILE = os.path.join(TESTS_PATH, "test.map")
TESTCONFIGFILE = os.path.join(TESTS_PATH, "mapserver-sample.conf")
XMARKS_IMAGE = os.path.join(TESTS_PATH, "xmarks.png")
HOME_IMAGE = os.path.join(TESTS_PATH, "home.png")
TEST_IMAGE = os.path.join(TESTS_PATH, "test.png")

INCOMING = tempfile.mkdtemp(prefix="mapscript")


# ==========================================================================
# Base testing classes


class MapPrimitivesTestCase(unittest.TestCase):
    """Base class for testing primitives (points, lines, shapes)
    in stand-alone mode"""

    def addPointToLine(self, line, point):
        """Using either the standard or next_generation_api"""
        try:
            line.add(point)
        except AttributeError:  # next_generation_api
            line.addPoint(point)
        except Exception:
            raise

    def getPointFromLine(self, line, index):
        """Using either the standard or next_generation_api"""
        try:
            point = line.get(index)
            return point
        except AttributeError:  # next_generation_api
            point = line.getPoint(index)
            return point
        except Exception:
            raise

    def addLineToShape(self, shape, line):
        """Using either the standard or next_generation_api"""
        try:
            shape.add(line)
        except AttributeError:  # next_generation_api
            shape.addLine(line)
        except Exception:
            raise

    def getLineFromShape(self, shape, index):
        """Using either the standard or next_generation_api"""
        try:
            line = shape.get(index)
            return line
        except AttributeError:  # next_generation_api
            line = shape.getLine(index)
            return line
        except Exception:
            raise

    def assertPointsEqual(self, first, second):
        self.assertAlmostEqual(first.x, second.x)
        self.assertAlmostEqual(first.y, second.y)

    def assertLinesEqual(self, first, second):
        assert first.numpoints == second.numpoints
        for i in range(first.numpoints):
            point_first = self.getPointFromLine(first, i)
            point_second = self.getPointFromLine(second, i)
            self.assertPointsEqual(point_first, point_second)

    def assertShapesEqual(self, first, second):
        assert first.numlines == second.numlines
        for i in range(first.numlines):
            line_first = self.getLineFromShape(first, i)
            line_second = self.getLineFromShape(second, i)
            self.assertLinesEqual(line_first, line_second)

    def assertRectsEqual(self, first, second):
        self.assertAlmostEqual(first.minx, second.minx)
        self.assertAlmostEqual(first.miny, second.miny)
        self.assertAlmostEqual(first.maxx, second.maxx)
        self.assertAlmostEqual(first.maxy, second.maxy)


class MapTestCase(MapPrimitivesTestCase):
    """Base class for testing with a map fixture"""

    def setUp(self):
        self.map = mapscript.mapObj(TESTMAPFILE)
        # self.xmarks_image = xmarks_image
        # self.test_image = test_image

    def tearDown(self):
        self.map = None


class MapZoomTestCase(MapPrimitivesTestCase):
    "Testing new zoom* methods that we are adapting from the PHP MapScript"

    def setUp(self):
        self.mapobj1 = mapscript.mapObj(TESTMAPFILE)
        # Change the extent for purposes of zoom testing
        rect = mapscript.rectObj()
        rect.minx, rect.miny, rect.maxx, rect.maxy = (-50.0, -50.0, 50.0, 50.0)
        self.mapobj1.extent = rect
        # Change height/width as well
        self.mapobj1.width, self.mapobj1.height = (100, 100)

    def tearDown(self):
        self.mapobj1 = None


class ShapeObjTestCase(MapPrimitivesTestCase):
    """Base class for shapeObj tests"""

    pass



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of colorObj
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import unittest

import mapscript


class ColorObjTestCase(unittest.TestCase):
    def testColorObjConstructorNoArgs(self):
        """a color can be initialized with no arguments"""
        c = mapscript.colorObj()
        assert (c.red, c.green, c.blue, c.alpha) == (0, 0, 0, 255)

    def testColorObjConstructorArgs(self):
        """a color can be initialized with arguments"""
        c = mapscript.colorObj(1, 2, 3, 4)
        assert (c.red, c.green, c.blue, c.alpha) == (1, 2, 3, 4)

    def testColorObjToHex(self):
        """a color can be outputted as hex"""
        c = mapscript.colorObj(255, 255, 255)
        assert c.toHex() == "#ffffff"

    def testColorObjToHexBadly(self):
        """raise an error in the case of an undefined color"""
        c = mapscript.colorObj(-1, -1, -1)
        self.assertRaises(mapscript.MapServerError, c.toHex)

    def testColorObjSetRGB(self):
        """a color can be set using setRGB method"""
        c = mapscript.colorObj()
        c.setRGB(255, 255, 255, 100)
        assert (c.red, c.green, c.blue, c.alpha) == (255, 255, 255, 100)

    def testColorObjSetHexLower(self):
        """a color can be set using lower case hex"""
        c = mapscript.colorObj()
        c.setHex("#ffffff64")
        assert (c.red, c.green, c.blue, c.alpha) == (255, 255, 255, 100)

    def testColorObjSetHexUpper(self):
        """a color can be set using upper case hex"""
        c = mapscript.colorObj()
        c.setHex("#FFFFFF")
        assert (c.red, c.green, c.blue) == (255, 255, 255)

    def testColorObjSetHexBadly(self):
        """invalid hex color string raises proper error"""
        c = mapscript.colorObj()
        self.assertRaises(mapscript.MapServerError, c.setHex, "#fffffg")


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of Map
# Author:   Umberto Nicoletti, unicoletti@prometeo.it
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import gc
import unittest

import mapscript

from .testing import TESTMAPFILE


class ParentReferenceTestCase(unittest.TestCase):
    def initMap(self):
        self.map = mapscript.mapObj(TESTMAPFILE)

    def testGetLayerObj(self):
        self.initMap()
        layer = self.map.getLayer(1)
        self.map = None
        assert str(layer.p_map).find("mapscript.mapObj") != -1
        gc.collect()
        assert layer.map is not None, layer.map

    def testGetLayerObjByName(self):
        self.initMap()
        layer = self.map.getLayerByName("POLYGON")
        self.map = None
        assert str(layer.p_map).find("mapscript.mapObj") != -1
        gc.collect()
        assert layer.map is not None, layer.map

    def testLayerObj(self):
        self.initMap()
        layer = mapscript.layerObj(self.map)
        self.map = None
        assert str(layer.p_map).find("mapscript.mapObj") != -1
        gc.collect()
        assert layer.map is not None, layer.map

    def testInsertLayerObj(self):
        self.initMap()
        layer = mapscript.layerObj()
        self.map.insertLayer(layer)
        self.map = None
        assert str(layer.p_map).find("mapscript.mapObj") != -1
        gc.collect()
        assert layer.map is not None, layer.map

    def testGetClassObj(self):
        self.initMap()
        layer = self.map.getLayer(1)
        clazz = layer.getClass(0)
        self.map = None
        layer = None
        assert str(clazz.p_layer).find("mapscript.layerObj") != -1
        gc.collect()
        assert clazz.layer is not None, clazz.layer

    def testClassObj(self):
        self.initMap()
        layer = mapscript.layerObj(self.map)
        clazz = mapscript.classObj(layer)
        self.map = None
        layer = None
        assert str(clazz.p_layer).find("mapscript.layerObj") != -1
        gc.collect()
        assert clazz.layer is not None, clazz.layer

    def testInsertClassObj(self):
        self.initMap()
        layer = mapscript.layerObj()
        clazz = mapscript.classObj()
        layer.insertClass(clazz)
        self.map = None
        layer = None
        assert str(clazz.p_layer).find("mapscript.layerObj") != -1
        gc.collect()
        assert clazz.layer is not None, clazz.layer

    def testRemoveClassObj(self):
        self.initMap()
        layer = self.map.getLayer(1)
        clazz = layer.removeClass(0)
        if hasattr(clazz, "p_layer"):
            assert clazz.p_layer is None, clazz.p_layer
        assert clazz.layer is None, clazz.layer
        self.initMap()
        position = self.map.getLayer(0).insertClass(clazz)
        assert position == 0, position
        assert clazz.p_layer is not None, clazz.p_layer
        assert clazz.layer is not None, clazz.layer


if __name__ == "__main__":
    unittest.main()
